/*
Copyright 2018 The Kubernetes Authors.

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
*/

package signers

import (
	"k8s.io/autoscaler/cluster-autoscaler/cloudprovider/alicloud/alibaba-cloud-sdk-go/sdk/auth/credentials"
)

// SignerV2 is wrapper of credential
type SignerV2 struct {
	credential *credentials.RsaKeyPairCredential
}

// GetExtraParam returns params
func (signer *SignerV2) GetExtraParam() map[string]string {
	return nil
}

// NewSignerV2 returns SignerV2
func NewSignerV2(credential *credentials.RsaKeyPairCredential) (*SignerV2, error) {
	return &SignerV2{
		credential: credential,
	}, nil
}

// GetName returns "SHA256withRSA"
func (*SignerV2) GetName() string {
	return "SHA256withRSA"
}

// GetType returns "PRIVATEKEY"
func (*SignerV2) GetType() string {
	return "PRIVATEKEY"
}

// GetVersion returns "1.0"
func (*SignerV2) GetVersion() string {
	return "1.0"
}

// GetAccessKeyId returns accessKeyId
func (signer *SignerV2) GetAccessKeyId() (accessKeyId string, err error) {
	return signer.credential.PublicKeyId, err
}

// Sign create signer
func (signer *SignerV2) Sign(stringToSign, secretSuffix string) string {
	secret := signer.credential.PrivateKey
	return Sha256WithRsa(stringToSign, secret)
}

// Shutdown doesn't implement
func (signer *SignerV2) Shutdown() {}
